class XmlParser
  class InvalidXML < StandardError; end

  VALID_XML_TAGS = %w[package to from subject body form]

  def self.parse_xml(raw_xml, validate_tags = false)
    if raw_xml.include?('DOCTYPE')
      Rails.logger.error("Invalid XML - DOCTYPE declaration detected")
      raise InvalidXML
    end
    if raw_xml.include?('!ENTITY')
      Rails.logger.error("Invalid XML - Entity injection attempt detected")
      raise InvalidXML
    end
    raw_xml =~ /^<\?xml.*\?>(.*)/m # workaround for wrong encoding in Outlook Add-In
    insecure_xml = Nokogiri::XML($1 || raw_xml) {|config| config.strict.nonoent}
    validate_xml(insecure_xml, validate_tags)
  end

  def self.validate_xml(insecure_xml, validate_tags)
    messages = []
    messages << 'Only one top-level node allowed' if insecure_xml.children.count > 1
    nodes = insecure_xml.search("*")
    if validate_tags
      messages << 'Top level node must be <package>' unless insecure_xml.child.name == 'package'
      nodes.map(&:name).each do |name| 
        messages << "Invalid tag: #{name}" if VALID_XML_TAGS.exclude?(name) 
      end
    end
    messages << 'Entity injection attempt detected' if nodes.map(&:child).map(&:class).include?(Nokogiri::XML::EntityReference)

    if messages.any?
      messages.each { |message| Rails.logger.error("Invalid XML - #{message}") }

      raise InvalidXML
    else
      insecure_xml
    end
  end
end
