namespace :aspera do
  desc "Renews secret yml and updates appropriate passwords."
  task :renew_secrets => :environment do
    new_secret = ActiveSupport::SecureRandom.hex(64)

    puts 'New secret auto generated successfully.'
    # Decrypt node password using old secret, update using new secret
    Node.all.each do |node|
      current_decrypted_pwd = node.unencrypted_api_password
      salt = node.salt
      if current_decrypted_pwd.present? && salt.present?
        new_encrypted_pwd = EzCrypto::Key.new(new_secret).encrypt64("#{current_decrypted_pwd}#{salt}")
        node.update_attribute(:encrypted_api_password, new_encrypted_pwd)
        node.save
        puts "Updating database record: Node config password updated successfully for host : #{node.host} & name : #{node.name}."
      end
    end

    # Decrypt SMTP password using old secret, update using new secret
    MConfiguration.all.each do |node|
      current_decrypted_pwd = node.unencrypted_smtp_password
      salt = node.salt
      if current_decrypted_pwd.present? && salt.present?
        new_encrypted_pwd = EzCrypto::Key.new(new_secret).encrypt64("#{current_decrypted_pwd}#{salt}")
        node.update_attribute(:encrypted_smtp_password, new_encrypted_pwd)
        node.save
        puts "Updating database record: SMTP config password updated successfully for server : #{node.m_smtp_server}."
      end
    end

    configuration = YAML::load(ERB.new(IO.read(Rails.configuration.database_configuration_file)).result)
    # Decrypt all passwords in database.yml using old secret, update using new secret
    configuration.keys.each do |env|
      password = configuration[env]['password']
      if password.present?
        begin
          current_encrypted_password = EzCrypto::Key.new(App.secret).decrypt64(password)
          configuration[env]['password'] = EzCrypto::Key.new(new_secret).encrypt64(current_encrypted_password).strip
          puts "Updating database.yml file: Mysql database password updated successfully for ENV : #{env}"
        rescue OpenSSL::Cipher::CipherError
          puts "Updating database.yml file: Failed to decrypt database password for #{env} environment. Not a problem if database passwords are not encrypted."
        end
      end
    end

    File.open(Rails.configuration.database_configuration_file, 'w') do |file|
      file.write(configuration.to_yaml)
    end

    puts 'Updating secret.yml file'
    File.open(File.join(RAILS_ROOT, 'config', 'secret.yml'), 'w') {|file| YAML.dump({"secret" => new_secret}, file)}
    puts 'Done'
  end
end