;(function ( $, window, undefined ) {
  "use strict";

  var pluginName     = 'contactable',
      pluginInstance = 0;

  function Contactable( element, options ) {
    this.element = element;
    this.uid     = pluginInstance++;

    var defaults = {
        button : {
          id: 'contacts_button_' + this.uid,
          text: navigator.userAgent.toUpperCase().indexOf('MSIE') >= 0 ? '&nbsp;+&nbsp;' : '+',
        },
        dialog : {
          id: 'contacts_dialog_' + this.uid
        },
        pagination : {
          id: 'pagination_container_' + this.uid,
          css_class: 'pagination_container_' + this.uid
        },
        filter : {
          id: 'filter-' + this.uid
        },
        current_recipients : {
          id: 'current_recipients_' + this.uid
        }
      };


    this.options = $.extend(true, {}, defaults, options);

    this._defaults = defaults;
    this._name     = pluginName;

    this.init();
  }

  Contactable.prototype = {

    init: function () {

      var $that  = $(this.element),
          that   = this,
          button = $('<button alt="Show contacts" class="contacts_button" style="vertical-align: top; padding-top: 4px; padding-bottom: 4px;" type="button" id="' + this.options.button.id + '">' + this.options.button.text + '</button>')
            .click(function(e) {
              e.preventDefault();

              // Elements
              var pagination = '<div class="' + that.options.pagination.css_class + '" id="' + that.options.pagination.id + '" style="position:absolute;top:-2px;left:355px;"></div>',
                  filter     = '<input type="text" id="' + that.options.filter.id + '" maxlength="255" placeholder="Filter&hellip;" style="position:absolute;top:5px;left:130px;">',
                  current_recipients = $('<div class="hide" id="' + that.options.current_recipients.id + '">').text($that.val());

              // Events
              $('body').on('click', '#' + that.options.pagination.id + ' li:not(.disabled) a', function () {
                $.ajax({
                  url: this.href,
                  dataType: "script",
                  data: {
                    current_recipients: $('#'+that.options.current_recipients.id).text()
                  }
                });
                return false;
              })
              .on('keyup', '#' + that.options.filter.id, _.debounce(function() {
                $.get( $('#filter_contacts_path').val(), {
                  uid                : that.uid,
                  filter             : $(this).val(),
                  current_recipients : $that.val()
                });
              }, 500));

              $('<div id="' + that.options.dialog.id + '"></div>')
                .baseDialog({
                  title: 'Contacts',
                  autoOpen: true,
                  open: function () {
                    toggleAsperaWebObjectVisibility();

                    if ( !$('#' + that.options.filter.id).length )            { $(this).after(filter); }
                    if ( !$('#' + that.options.pagination.id).length )        { $(this).after(pagination); }
                    if ( !$('#' + that.options.current_recipients.id).length) { $(this).after(current_recipients); }

                    $.get($that.data('browse_contacts_path'), { uid : that.uid,
                                                                current_recipients: $('#'+that.options.current_recipients.id).text(),
                                                                filter: $('#'+that.options.filter.id).val() } );

                  },
                  buttons: {
                    Select: {
                      text: 'Select',
                      id: 'select_button',
                      click: function() {
                        that._replaceRecipients ( $that, $('#' + that.options.current_recipients.id).text() );

                        // Wait to trigger autosize for slower browsers to populate the input
                        window.setTimeout(function(){
                          $that.trigger('autosize');
                        }, 200);

                        $(this).dialog('close');
                      }
                    },
                    Cancel: {
                      text: 'Cancel',
                      id: 'cancel_button',
                      click: function () { $(this).dialog('close'); }
                    }
                  },
                  close: function () {
                    $(this).remove();
                    toggleAsperaWebObjectVisibility();
                  }
                });
            });

          $that
            .after(button)
            // Don't navigate away from the field on tab when selecting an item.
            .bind( "keydown", function (e) {
              if ( e.keyCode === $.ui.keyCode.TAB && $(this).data('ui-autocomplete').menu.active ) {
                e.preventDefault();
              }
            })
            .autocomplete({
              autoFocus: true,
              source: function ( request, response) {
                var inputID = this.element.attr('id');
                $.getJSON($('#' + inputID).data('autocomplete_source'), {
                  term : that._extractLastRecipient( request.term )
                }, response );
              },
              search: function () {
                // custom minLength on last element
                var term = that._extractLastRecipient(this.value);
                if (term.length < 2) { return false; }
              },
              select: function (e, ui) {
                that._addRecipient($that, ui.item.name);

                // Wait to trigger autosize for slower browsers to populate the input
                window.setTimeout(function(){
                  $that.trigger('autosize');
                }, 200);

                return false;
              },
              focus: function () { return false; } // Prevent value inserted on focus
            })
            .data( 'ui-autocomplete' )._renderItem = function( ul, item ) {
              return $( '<li></li>' )
                .data( 'ui-autocomplete-item', item)
                .append('<a>' + '<strong>' + item.name + '</strong> ' + item.informal + '</a>')
                .appendTo(ul);
            };

          $('body').on('click', '#contacts tbody tr', function(e) {
             var checkedElement;
            if (e.target.type !== 'checkbox') {
              checkedElement = $(':checkbox', this).attr('checked', function() { return !this.checked; });
            } else {
              checkedElement = e.target;
            }

            that._toggleName(checkedElement, this, '#' + that.options.current_recipients.id);
          });

          $('#' + that.options.dialog.id + ' tbody tr.selected :checkbox').prop('checked', true);

          return this;
    },

    _addRecipient : function (element, name) {
      var terms = this._splitRecipients( element.val() );
      terms.pop();
      terms.push( this._htmlDecode(name) );
      terms.push( '' );
      element.val(terms.join( ', ' ));
    },
    _htmlDecode : function ( input ) {
      var e = document.createElement('div');
      e.innerHTML = input;
      return e.childNodes.length === 0 ? "" : e.childNodes[0].nodeValue;
    },
    _extractLastRecipient : function ( term ) { return this._splitRecipients( term ).pop(); },
    _replaceRecipients : function ( element, names ) {
      element.val( this._splitRecipients(names).join( ', ') );
    },
    _splitRecipients : function ( val ) { return $.grep(val.split( /[,;]\s*/ ), function (n) { return(n); }); },

    _toggleName : function ( checkedElement, row, currentRecipientsId ) {
      var names = this._splitRecipients( $(currentRecipientsId).text() );
      var name  = $(checkedElement).attr('data-name'); // use attr() since data() coerces values

      if ( $(checkedElement).is(':checked') ) {
        names.push(name);
        $(currentRecipientsId).text(names.join(', '));
      } else {
        $(currentRecipientsId).text(_.without(names, name ).join(', '));
      }
      $(row).toggleClass('selected');
    }
  };

  $.fn[pluginName] = function ( options ) {
    return this.each(function () {
      if (!$.data(this, 'plugin_' + pluginName)) {
        $.data(this, 'plugin_' + pluginName, new Contactable( this, options ));
      }
    });
  };

})(jQuery, window);
